
package com.fragments;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.okbuddy.store.Order;
import com.okbuddy.store.OrderAdapter;
import com.okbuddy.store.R;

import java.util.ArrayList;
import java.util.List;

public class PreparingFragment extends Fragment {

    private RecyclerView recyclerView;
    private List<Order> orderList;
    private OrderAdapter orderAdapter;
    private DatabaseReference databaseReference;
    private String currentCompanyId;  // This will be set from the arguments
    private String orderStatus = "Preparing";  // This fragment is for "Preparing" orders

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_order_status, container, false);

        // Retrieve the currentCompanyId from the arguments
        if (getArguments() != null) {
            currentCompanyId = getArguments().getString("currentCompanyId");
        }

        recyclerView = view.findViewById(R.id.recyclerView);
        recyclerView.setHasFixedSize(true);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));

        orderList = new ArrayList<>();
        orderAdapter = new OrderAdapter(getContext(), orderList);
        recyclerView.setAdapter(orderAdapter);

        // Initialize Firebase Database
        databaseReference = FirebaseDatabase.getInstance("https://ersinfo-367917-default-rtdb.firebaseio.com/").getReference("qrorders");

        // Create a query that filters orders by the current company ID and status
        Query query = databaseReference.orderByChild("OrderStatus").equalTo(orderStatus);

        // Read orders from Firebase
        query.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                orderList.clear();
                for (DataSnapshot orderSnapshot : dataSnapshot.getChildren()) {
                    Order order = orderSnapshot.getValue(Order.class);
                    if (order != null && order.getICompanyId().equals(currentCompanyId)) {
                        order.setFirebaseKey(orderSnapshot.getKey());  // Set the Firebase key
                        orderList.add(order);
                    }
                }
                orderAdapter.notifyDataSetChanged();
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {
                Toast.makeText(getContext(), "Failed to load orders: " + databaseError.getMessage(), Toast.LENGTH_SHORT).show();
            }
        });

        return view;
    }
}
