package com.okbuddy.store;

import android.annotation.SuppressLint;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import java.lang.reflect.Type;
import java.util.List;

public class OrderAdapter extends RecyclerView.Adapter<OrderAdapter.OrderViewHolder> {

    private final Context context;
    private final List<Order> orderList;
    private final DatabaseReference databaseReference;

    public OrderAdapter(Context context, List<Order> orderList) {
        this.context = context;
        this.orderList = orderList;
        this.databaseReference = FirebaseDatabase.getInstance("https://ersinfo-367917-default-rtdb.firebaseio.com/").getReference("qrorders");
    }

    @NonNull
    @Override
    public OrderViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.order_item, parent, false);
        return new OrderViewHolder(view);
    }

    @SuppressLint("SetTextI18n")
    @Override
    public void onBindViewHolder(@NonNull OrderViewHolder holder, int position) {
        Order order = orderList.get(position);

        holder.tvTableNumber.setText("Table: " + order.getTableName());
        holder.orderStatus.setText("Status: " + order.getOrderStatus());

        // Parse the JSON string into a list of items
        Type type = new TypeToken<List<Item>>(){}.getType();
        List<Item> items = new Gson().fromJson(order.getCartItems(), type);

        // Clear the LinearLayout (in case it has old views)
        holder.itemListLayout.removeAllViews();

        // Dynamically add each item to the LinearLayout
        for (Item item : items) {
            TextView textView = new TextView(context);
            textView.setText(String.format("Item: %s, Price: %s, Quantity: %s", item.vItemType, item.fDiscountPrice, item.Qty));
            holder.itemListLayout.addView(textView);
        }

        holder.btnPreparing.setOnClickListener(v -> updateOrderStatus(order, "Preparing"));
        holder.btnReady.setOnClickListener(v -> OrderAdapter.this.updateOrderStatus(order, "Ready"));
        holder.btnGiven.setOnClickListener(v -> updateOrderStatus(order, "Given"));

    }


    @Override
    public int getItemCount() {
        return orderList.size();
    }
    private void updateOrderStatus(Order order, String newStatus) {
        DatabaseReference orderRef = databaseReference.child(order.getFirebaseKey());  // Use the Firebase key
        orderRef.child("OrderStatus").setValue(newStatus);
    }

    public static class OrderViewHolder extends RecyclerView.ViewHolder {
        TextView tvTableNumber, orderStatus;
        LinearLayout itemListLayout;
        Button btnPreparing, btnReady, btnGiven;

        public OrderViewHolder(@NonNull View itemView) {
            super(itemView);
            tvTableNumber = itemView.findViewById(R.id.tvTableNumber);
            orderStatus = itemView.findViewById(R.id.orderStatus);  // Make sure this ID exists in your XML
            itemListLayout = itemView.findViewById(R.id.itemListLayout);  // Make sure this ID exists in your XML
            btnPreparing = itemView.findViewById(R.id.btnPreparing);
            btnReady = itemView.findViewById(R.id.btnReady);
            btnGiven = itemView.findViewById(R.id.btnGiven);
        }
    }
}