package com.okbuddy.store;

import android.os.Bundle;
import android.widget.Toast;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.google.firebase.database.*;
import com.okbuddy.store.Order;
import com.okbuddy.store.OrderAdapter;

import java.util.ArrayList;
import java.util.List;

public class PreparingActivity extends AppCompatActivity {

    private RecyclerView recyclerView;
    private List<Order> orderList;
    private OrderAdapter orderAdapter;
    private DatabaseReference databaseReference;
    private String currentCompanyId;
    private String orderStatus = "Preparing";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_preparing);

        // Get the currentCompanyId from the intent extras
        currentCompanyId = getIntent().getStringExtra("currentCompanyId");

        recyclerView = findViewById(R.id.recyclerView);
        recyclerView.setHasFixedSize(true);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        orderList = new ArrayList<>();
        orderAdapter = new OrderAdapter(this, orderList);
        recyclerView.setAdapter(orderAdapter);

        // Initialize Firebase Database
        databaseReference = FirebaseDatabase.getInstance("https://ersinfo-367917-default-rtdb.firebaseio.com/").getReference("qrorders");

        // Create a query that filters orders by the current company ID and status
        Query query = databaseReference.orderByChild("iCompanyId").equalTo(currentCompanyId);

        // Read orders from Firebase
        query.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                orderList.clear();
                for (DataSnapshot orderSnapshot : dataSnapshot.getChildren()) {
                    Order order = orderSnapshot.getValue(Order.class);
                    if (order != null && order.getOrderStatus().equals(orderStatus)) {
                        order.setFirebaseKey(orderSnapshot.getKey());
                        orderList.add(order);
                    }
                }
                orderAdapter.notifyDataSetChanged();
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {
                Toast.makeText(PreparingActivity.this, "Failed to load orders: " + databaseError.getMessage(), Toast.LENGTH_SHORT).show();
            }
        });
    }
}
