package com.okbuddy.store;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;
import android.media.MediaPlayer;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;



public class ScratchCardView extends View {
    private Bitmap bitmap;
    private Canvas bitmapCanvas;
    private Path path;
    private Paint bitmapPaint;
    private Paint pathPaint;
    private ImageView rewardImage;
    private float revealPercentage;
    private MediaPlayer scratchSoundPlayer;
    private MediaPlayer rewardSoundPlayer;


    public ScratchCardView(Context context, AttributeSet attrs) {
        super(context, attrs);
        path = new Path();
        bitmapPaint = new Paint(Paint.DITHER_FLAG);
        pathPaint = new Paint();
        pathPaint.setAntiAlias(true);
        pathPaint.setDither(true);
        pathPaint.setColor(Color.TRANSPARENT);
        pathPaint.setStyle(Paint.Style.STROKE);
        pathPaint.setStrokeJoin(Paint.Join.ROUND);
        pathPaint.setStrokeCap(Paint.Cap.ROUND);
        pathPaint.setXfermode(new PorterDuffXfermode(PorterDuff.Mode.CLEAR));
        pathPaint.setStrokeWidth(50);
        
        scratchSoundPlayer = MediaPlayer.create(context, R.raw.scratch_sound);
        rewardSoundPlayer = MediaPlayer.create(context, R.raw.reward_sound);
    }

    public void setRevealListener(ImageView rewardImage, float revealPercentage) {
        this.rewardImage = rewardImage;
        this.revealPercentage = revealPercentage;
        rewardImage.setVisibility(INVISIBLE);
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        super.onSizeChanged(w, h, oldw, oldh);
        bitmap = Bitmap.createBitmap(w, h, Bitmap.Config.ARGB_8888);
        bitmapCanvas = new Canvas(bitmap);
        bitmapCanvas.drawColor(Color.GRAY);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        canvas.drawBitmap(bitmap, 0, 0, bitmapPaint);
        bitmapCanvas.drawPath(path, pathPaint);
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_DOWN || event.getAction() == MotionEvent.ACTION_MOVE) {
            if (!scratchSoundPlayer.isPlaying()) {
                scratchSoundPlayer.start();
            }
        } else {
            scratchSoundPlayer.pause();
            scratchSoundPlayer.seekTo(0);
        }

        float x = event.getX();
        float y = event.getY();

        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                path.moveTo(x, y);
                break;
            case MotionEvent.ACTION_MOVE:
                path.lineTo(x, y);
                break;
            case MotionEvent.ACTION_UP:
                break;
        }

        invalidate();
        checkRevealPercentage();
        return true;
    }

    private void checkRevealPercentage() {
        int[] location = new int[2];
        getLocationOnScreen(location);
        int x = location[0];
        int y = location[1];

        Bitmap bitmap = Bitmap.createBitmap(getWidth(), getHeight(), Bitmap.Config.ARGB_8888);
        draw(new Canvas(bitmap));

        int totalPixels = bitmap.getWidth() * bitmap.getHeight();
        int scratchedPixels = 0;

        for (int i = 0; i < bitmap.getWidth(); i++) {
            for (int j = 0; j < bitmap.getHeight(); j++) {
                if (bitmap.getPixel(i, j) == Color.TRANSPARENT) {
                    scratchedPixels++;
                }
            }
        }

        float percentage = (float) scratchedPixels / totalPixels * 100;

        if (percentage >= revealPercentage && rewardImage != null) {
            if (rewardImage.getVisibility() != VISIBLE) {
                rewardSoundPlayer.start();
                Animation animation = AnimationUtils.loadAnimation(getContext(), R.anim.reward_animation);
                rewardImage.startAnimation(animation);
            }
            rewardImage.setVisibility(VISIBLE);
        }

    }
}
