package com.okbuddy.store;

import android.os.Bundle;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import com.general.files.GeneralFunctions;
import java.util.ArrayList;
import java.util.List;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

public class TableOrderActivity extends AppCompatActivity {

    private RecyclerView recyclerView;
    private List<Order> orderList;
    private OrderAdapter orderAdapter;
    private DatabaseReference databaseReference;
    private GeneralFunctions generalFunc;
    private String currentCompanyId;
    private TextView tvNoOrders; // TextView for displaying "No orders" message
    private ProgressBar progressBar; // ProgressBar for loading indication

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_table_order);

        generalFunc = new GeneralFunctions(this);
        currentCompanyId = generalFunc.getMemberId();

        recyclerView = findViewById(R.id.recyclerView);
        tvNoOrders = findViewById(R.id.tvNoOrders); // Initialize TextView
        progressBar = findViewById(R.id.progressBar); // Initialize ProgressBar

        recyclerView.setHasFixedSize(true);
        recyclerView.setLayoutManager(new LinearLayoutManager(this));

        orderList = new ArrayList<>();
        orderAdapter = new OrderAdapter(this, orderList);
        recyclerView.setAdapter(orderAdapter);

        databaseReference = FirebaseDatabase.getInstance("https://ersinfo-367917-default-rtdb.firebaseio.com/").getReference("qrorders");
        Query query = databaseReference.orderByChild("iCompanyId").equalTo(currentCompanyId);

        // Display ProgressBar while loading
        progressBar.setVisibility(View.VISIBLE);

        query.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot dataSnapshot) {
                orderList.clear();
                for (DataSnapshot orderSnapshot : dataSnapshot.getChildren()) {
                    Order order = orderSnapshot.getValue(Order.class);
                    if (order != null && "Unpaid".equals(order.getOrderStatus())) {
                        order.setFirebaseKey(orderSnapshot.getKey());
                        orderList.add(order);
                    }
                }
                progressBar.setVisibility(View.GONE); // Hide ProgressBar after loading

                // Display "No orders" message if orderList is empty
                if (orderList.isEmpty()) {
                    tvNoOrders.setVisibility(View.VISIBLE);
                } else {
                    tvNoOrders.setVisibility(View.GONE);
                }

                orderAdapter.notifyDataSetChanged();
            }

            @Override
            public void onCancelled(@NonNull DatabaseError databaseError) {
                progressBar.setVisibility(View.GONE); // Hide ProgressBar on error
                Toast.makeText(TableOrderActivity.this, "Failed to load orders: " + databaseError.getMessage(), Toast.LENGTH_SHORT).show();
            }
        });
    }
}